/*
 * The FUJABA ToolSuite project:
 *
 *   FUJABA is the acronym for 'From Uml to Java And Back Again'
 *   and originally aims to provide an environment for round-trip
 *   engineering using UML as visual programming language. During
 *   the last years, the environment has become a base for several
 *   research activities, e.g. distributed software, database
 *   systems, modelling mechanical and electrical systems and
 *   their simulation. Thus, the environment has become a project,
 *   where this source code is part of. Further details are avail-
 *   able via http://www.fujaba.de
 *
 *      Copyright (C) Fujaba Development Group
 *
 *   This library is free software; you can redistribute it and/or
 *   modify it under the terms of the GNU Lesser General Public
 *   License as published by the Free Software Foundation; either
 *   version 2.1 of the License, or (at your option) any later version.
 *
 *   You should have received a copy of the GNU Lesser General Public
 *   License along with this library; if not, write to the Free
 *   Software Foundation, Inc., 59 Temple Place, Suite 330, Boston,
 *   MA 02111-1307, USA or download the license under
 *   http://www.gnu.org/copyleft/lesser.html
 *
 * WARRANTY:
 *
 *   This library is distributed in the hope that it will be useful,
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
 *   GNU Lesser General Public License for more details.
 *
 * Contact address:
 *
 *   Fujaba Management Board
 *   Software Engineering Group
 *   University of Paderborn
 *   Warburgerstr. 100
 *   D-33098 Paderborn
 *   Germany
 *
 *   URL  : http://www.fujaba.de
 *   email: info@fujaba.de
 *
 */
package de.uni_paderborn.fujaba.fsa.listener;

import java.awt.*;
import java.awt.event.MouseEvent;
import java.util.Iterator;
import java.util.Vector;

import javax.swing.*;

import de.tu_bs.coobra.*;
import de.uni_paderborn.fujaba.asg.ASGElement;
import de.uni_paderborn.fujaba.fsa.FSAObject;
import de.uni_paderborn.fujaba.fsa.SelectionManager;
import de.uni_paderborn.fujaba.fsa.swing.LayerManager;


/**
 * Standard mouselistener for Components displaying DiagramItems</p> Handles dragging and selecting
 * of Objects as well as putting selected Objects on top of others
 *
 * @author    $Author: cschneid $
 * @version   $Revision: 1.21 $
 */
public class DragMouseListener extends ComponentMouseListener implements AscendDescendMouseHandler.Descend
{
   /**
    * The default constructor is empty.
    */
   protected DragMouseListener() { }


   /**
    * No comment provided by developer, please add a comment to improve documentation.
    */
   private static DragMouseListener singleton = null;


   /**
    * No comment provided by developer, please add a comment to improve documentation.
    *
    * @return   No description provided
    */
   public static DragMouseListener get()
   {
      if (singleton == null)
      {
         singleton = new DragMouseListener();
      }

      return singleton;
   }


   /**
    * Inner attribute for calculating a mouse move.
    */
   private Point lastPoint = null;

   /**
    * No comment provided by developer, please add a comment to improve documentation.
    */
   protected boolean dragging = false;


   /**
    * Overwrites the mouseClicked method of ComponentMouseListener. It sets the lastPoint attribute.
    *
    * @param event  No description provided
    */
   public void mouseClicked (MouseEvent event)
   {
      if (event.getClickCount() == 1)
      {
         lastPoint = event.getPoint();
      }
   }


   /**
    * Overwrites the mousePressed method of ComponentMouseListener. It sets the lastPoint attribute.
    *
    * @param event  No description provided
    */
   public void mousePressed (MouseEvent event)
   {
      if (event.getClickCount() == 1)
      {
         JComponent comp = (JComponent) event.getSource();
         setOrientation (comp, event);
         lastComponent = comp;
         lastPoint = event.getPoint();
         event.consume();
      }
   }


   /**
    * Overwrites the mouseReleased method of ComponentMouseListener. It sets the lastPoint
    * attribute to null.
    *
    * @param event  No description provided
    */
   public void mouseReleased (MouseEvent event)
   {
      JComponent comp = (JComponent) event.getSource();
      ObjectChangeCause saveLocationsCause = new ObjectChangeStringCause ("drag");
      ObjectChange.pushCause (saveLocationsCause);
      try
      {
         Iterator iter = SelectionManager.get().iteratorOfSelectionAsIncrements();
         while (iter.hasNext())
         {
            ASGElement element = (ASGElement) iter.next();
            element.saveFSAProperties();
         }
      }
      finally
      {
         ObjectChange.popCause (saveLocationsCause);
      }

      if (dragging)
      {
         setDragging (false);
         Container parent = comp.getParent();
         if (parent != null && parent instanceof JLayeredPane)
         {
            JLayeredPane pane = (JLayeredPane) parent;

            pane.setPosition (comp, 0);
            pane.revalidate();
            pane.repaint();
         }
      }
      lastPoint = null;
      lastComponent = null;
      event.consume();
   }


   /**
    * Overwrites the mouseClicked method of ComponentMouseListener. It moves the event's source
    * component to the new calculated position.
    *
    * @param event  No description provided
    */
   public void mouseDragged (MouseEvent event)
   {
      JComponent source = (JComponent) event.getSource();

      Point eventPoint = event.getPoint();

      if (getOrientation() == INNER)
      {
         SelectionManager manager = SelectionManager.get();

         if (!dragging)
         {
            setDragging (true);
         }

         if (lastPoint == null)
         {
            lastPoint = eventPoint;
         }

         int deltaX = (int)  (eventPoint.getX() - lastPoint.getX());
         int deltaY = (int)  (eventPoint.getY() - lastPoint.getY());

         boolean sourceFound = false;

         JComponent comp;

         Iterator selIter = manager.iteratorOfSelection();
         while (selIter.hasNext())
         {
            FSAObject object = (FSAObject) selIter.next();
            FSAObject parent = object.getParent();
            if (parent == null || !parent.isSelected())
            {
               comp = object.getJComponent();
               drag (comp, deltaX, deltaY);

               if (comp == source)
               {
                  sourceFound = true;
               }
            }
         }
         if (!sourceFound)
         {
            drag (source, deltaX, deltaY);
         }
         event.consume();
      }
   }


   /**
    * No comment provided by developer, please add a comment to improve documentation.
    *
    * @param comp    No description provided
    * @param deltaX  No description provided
    * @param deltaY  No description provided
    */
   protected void drag (JComponent comp, int deltaX, int deltaY)
   {
      Point point = comp.getLocation();
      point.x += deltaX;
      if (point.x < 0)
      {
         point.x = 0;
      }
      point.y += deltaY;
      if (point.y < 0)
      {
         point.y = 0;
      }
      if (point.x < 0)
      {
         lastPoint.x += point.x;
      }
      if (point.y < 0)
      {
         lastPoint.y += point.y;
      }
      comp.setLocation (point);

      JComponent parent = (JComponent) comp.getParent();
      if (parent != null)
      {
         parent.revalidate();
      } // end of if ()
   }


   /**
    * If drag is true move all selected components to the drag layer. If drag is false move
    * all selected components to the default layer. Repaint all modified LayeredPanes afterwards.
    *
    * @param drag  The new dragging value
    */
   protected void setDragging (boolean drag)
   {
      if (this.dragging != drag)
      {
         Vector panes = new Vector();
         Iterator selIter = SelectionManager.get().iteratorOfSelection();
         int layer;
         while (selIter.hasNext())
         {
            FSAObject selected = (FSAObject) selIter.next();
            JComponent selectedComp = selected.getJComponent();
            Container parent;

            if (selectedComp != null &&
                (parent = selectedComp.getParent()) != null &&
               parent instanceof JLayeredPane)
            {
               JLayeredPane pane = (JLayeredPane) parent;

               if (drag)
               {
                  layer = LayerManager.get().getDragLayer (selectedComp);
                  pane.setLayer (selectedComp, layer);
               }
               else
               {
                  layer = LayerManager.get().getLayer (selectedComp);
                  pane.setLayer (selectedComp, layer);
                  pane.setPosition (selectedComp, 0);
               }

               panes.addElement (pane);
            }
         }
         Iterator paneIter = panes.iterator();
         while (paneIter.hasNext())
         {
             ((JLayeredPane) paneIter.next()).repaint();
         }
         dragging = drag;
      }
   } // selectedComp


   /**
    * Sets the orientation attribute of the DragMouseListener object
    *
    * @param comp   The new orientation value
    * @param event  The new orientation value
    */
   protected void setOrientation (Component comp, MouseEvent event)
   {
      super.setOrientation (comp, event);
      int orientation = getOrientation();

      if ( ( (orientation &  (EAST | WEST)) != 0) && comp.getWidth() <= 4 * DISTANCE)
      {
         orientation = orientation & ~ (EAST | WEST);
      }
      if ( ( (orientation &  (NORTH | SOUTH)) != 0) && comp.getHeight() <= 4 * DISTANCE)
      {
         orientation = orientation & ~ (NORTH | SOUTH);
      }
      if (orientation == 0)
      {
         orientation = INNER;
      }
      setOrientation (orientation);
   }
}

/*
 * $Log: DragMouseListener.java,v $
 * Revision 1.21  2005/01/21 12:16:57  cschneid
 * undo/redo move shown immediately, class chooser caches folders too
 *
 */
