/*
 * The FUJABA ToolSuite project:
 *
 *   FUJABA is the acronym for 'From Uml to Java And Back Again'
 *   and originally aims to provide an environment for round-trip
 *   engineering using UML as visual programming language. During
 *   the last years, the environment has become a base for several
 *   research activities, e.g. distributed software, database
 *   systems, modelling mechanical and electrical systems and
 *   their simulation. Thus, the environment has become a project,
 *   where this source code is part of. Further details are avail-
 *   able via http://www.fujaba.de
 *
 *      Copyright (C) 1997-2004 Fujaba Development Group
 *
 *   This library is free software; you can redistribute it and/or
 *   modify it under the terms of the GNU Lesser General Public
 *   License as published by the Free Software Foundation; either
 *   version 2.1 of the License, or (at your option) any later version.
 *
 *   You should have received a copy of the GNU Lesser General Public
 *   License along with this library; if not, write to the Free
 *   Software Foundation, Inc., 59 Temple Place, Suite 330, Boston,
 *   MA 02111-1307, USA or download the license under
 *   http://www.gnu.org/copyleft/lesser.html
 *
 * WARRANTY:
 *
 *   This library is distributed in the hope that it will be useful,
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
 *   GNU Lesser General Public License for more details.
 *
 * Contact adress:
 *
 *   Fujaba Management Board
 *   Software Engineering Group
 *   University of Paderborn
 *   Warburgerstr. 100
 *   D-33098 Paderborn
 *   Germany
 *
 *   URL  : http://www.fujaba.de
 *   email: info@fujaba.de
 *
 */
package de.uni_paderborn.fujaba.usecase;

import java.util.Iterator;

import de.uni_paderborn.fujaba.asg.ASGElement;
import de.uni_paderborn.fujaba.uml.UMLDiagram;
import de.uni_paderborn.fujaba.uml.UMLProject;
import de.upb.tools.fca.FEmptyIterator;
import de.upb.tools.fca.FPropHashSet;


/**
 * A Use Case Diagram. You can add Actors and Use Cases to it and let the Actors use the Use
 * Cases. Every Use Case Diagram contains a System-box. All Use Cases are placed therein.
 *
 * @author    $Author: schneider $
 * @version   $Revision: 1.25 $
 * @see       Usecase
 * @see       UsecaseActor
 * @see       UsecaseUsage
 * @see       UsecaseSystem
 */
public class UsecaseDiagram extends UMLDiagram
{
   /**
    * No comment provided by developer, please add a comment to improve documentation.
    */
   public void init()
   {
      // add System-box and a Use Case
      UsecaseSystem system = new UsecaseSystem (getName(), this);

      Usecase usecase = new Usecase();
      usecase.setName ("Use Case");
      system.getSystemDiagram().addToElements (usecase);
   }



   /**
    * Constructor for class UsecaseDiagram
    */
   public UsecaseDiagram()
   {
      super();
   }


   /**
    * Constructor for class UsecaseDiagram
    *
    * @param name     No description provided
    * @param project  No description provided
    */
   public UsecaseDiagram (String name, UMLProject project)
   {
      super (name, project);
   }



   /**
    * Returns a correct name for the tree.
    *
    * @return   No description provided
    */
   public String toString()
   {
      return this.getName();
   }


   /**
    * Sets the name attribute of the UsecaseDiagram object
    *
    * @param name  The new name value
    */
   public void setName (String name)
   {
      super.setName (name);
      UsecaseSystem system = getUsecaseSystem();
      if (system != null)
      {
         system.setName (name);
      }
   }



   /**
    * Returns this diagram's System-box.
    *
    * @return   this diagram's {@link UsecaseSystem}.
    */
   public UsecaseSystem getUsecaseSystem()
   {
      UsecaseSystem usecaseSystem = null;

      Iterator iter = iteratorOfElements();
      while (iter.hasNext() && usecaseSystem == null)
      {
         ASGElement asgElement = (ASGElement) iter.next();
         if (asgElement instanceof UsecaseSystem)
         {
            usecaseSystem = (UsecaseSystem) asgElement;
         }
      }
      return usecaseSystem;
   }


   /**
    * <pre>
    *          n          usecases           0..1
    * Usecase ------------------------------------ UsecaseDiagram
    *          usecases      parentUsecaseDiagram
    * </pre>
    */
   private FPropHashSet usecases;


   /**
    * No comment provided by developer, please add a comment to improve documentation.
    *
    * @return   No description provided
    */
   public int sizeOfUsecases()
   {
      return  ( (this.usecases == null)
         ? 0
         : this.usecases.size());
   }


   /**
    * No comment provided by developer, please add a comment to improve documentation.
    *
    * @param value  No description provided
    * @return       No description provided
    */
   public boolean removeFromUsecases (Usecase value)
   {
      boolean changed = false;
      if ( (this.usecases != null) &&  (value != null))
      {
         changed = this.usecases.remove (value);
         if (changed)
         {
            value.setParentUsecaseDiagram (null);
         }
      }
      return changed;
   }


   /**
    * No comment provided by developer, please add a comment to improve documentation.
    */
   public void removeAllFromUsecases()
   {
      Usecase tmpValue;
      Iterator iter = this.iteratorOfUsecases();
      while (iter.hasNext())
      {
         tmpValue = (Usecase) iter.next();
         this.removeFromUsecases (tmpValue);
      }
   }


   /**
    * No comment provided by developer, please add a comment to improve documentation.
    *
    * @return   No description provided
    */
   public Iterator iteratorOfUsecases()
   {
      return  ( (this.usecases == null)
         ? FEmptyIterator.get()
         : this.usecases.iterator());
   }


   /**
    * No comment provided by developer, please add a comment to improve documentation.
    *
    * @param value  No description provided
    * @return       No description provided
    */
   public boolean hasInUsecases (Usecase value)
   {
      return  ( (this.usecases != null) &&
          (value != null) &&
         this.usecases.contains (value));
   }


   /**
    * Access method for an one to n association.
    *
    * @param value  The object added.
    * @return       No description provided
    */
   public boolean addToUsecases (Usecase value)
   {
      boolean changed = false;
      if (value != null)
      {
         if (this.usecases == null)
         {
            this.usecases = new FPropHashSet (this, "usecases"); // or FTreeSet () or FLinkedList ()
         }
         changed = this.usecases.add (value);
         if (changed)
         {
            value.setParentUsecaseDiagram (this);
         }
      }
      return changed;
   }


   /**
    * No comment provided by developer, please add a comment to improve documentation.
    *
    * @return   No description provided
    */
   public boolean hasOnlyNewDisplayStructures()
   {
      return true;
   }


   /**
    * UMLMethod: '+ removeYou () : Void'
    */
   public void removeYou()
   {
      this.removeAllFromUsecases();
   }


   /**
    * Searches the ASG tree for a given id
    *
    * @param id  The id to search for.
    * @return    The Element with the given id, null if not found.
    */
   public ASGElement searchID (String id)
   {
      ASGElement elem = super.searchID (id);
      UsecaseSystem system = getUsecaseSystem();
      if ( (elem == null) &&  (system != null))
      {
         elem = system.searchID (id);
      }
      return elem;
   }
}

/*
 * $Log: UsecaseDiagram.java,v $
 * Revision 1.25  2004/10/20 17:50:28  schneider
 * Introduction of interfaces for class diagram classes
 *
 */
