/*
 * The FUJABA ToolSuite project:
 *
 *   FUJABA is the acronym for 'From Uml to Java And Back Again'
 *   and originally aims to provide an environment for round-trip
 *   engineering using UML as visual programming language. During
 *   the last years, the environment has become a base for several
 *   research activities, e.g. distributed software, database
 *   systems, modelling mechanical and electrical systems and
 *   their simulation. Thus, the environment has become a project,
 *   where this source code is part of. Further details are avail-
 *   able via http://www.fujaba.de
 *
 *      Copyright (C) 1997-2004 Fujaba Development Group
 *
 *   This library is free software; you can redistribute it and/or
 *   modify it under the terms of the GNU Lesser General Public
 *   License as published by the Free Software Foundation; either
 *   version 2.1 of the License, or (at your option) any later version.
 *
 *   You should have received a copy of the GNU Lesser General Public
 *   License along with this library; if not, write to the Free
 *   Software Foundation, Inc., 59 Temple Place, Suite 330, Boston,
 *   MA 02111-1307, USA or download the license under
 *   http://www.gnu.org/copyleft/lesser.html
 *
 * WARRANTY:
 *
 *   This library is distributed in the hope that it will be useful,
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
 *   GNU Lesser General Public License for more details.
 *
 * Contact adress:
 *
 *   Fujaba Management Board
 *   Software Engineering Group
 *   University of Paderborn
 *   Warburgerstr. 100
 *   D-33098 Paderborn
 *   Germany
 *
 *   URL  : http://www.fujaba.de
 *   email: info@fujaba.de
 *
 */
package de.uni_paderborn.fujaba.usecase;

import java.util.Iterator;

import de.uni_paderborn.fujaba.asg.ASGDiagram;
import de.uni_paderborn.fujaba.asg.ASGElement;
import de.uni_paderborn.fujaba.fsa.*;
import de.uni_paderborn.fujaba.fsa.unparse.UnparseManager;
import de.uni_paderborn.fujaba.uml.*;


/**
 * Some useful functions.
 *
 * @author    $Author: schneider $
 * @version   $Revision: 1.15 $
 */
public class Utility
{
   /**
    * Get the item's main FSAContainer. The container is identified by the propertyName defined
    * int the UnparseModule (default: entry).
    *
    * @param item  This UMLDiagramItem's main FSAContainer is returned.
    * @return      The item's main FSAContainer.
    * @see         UMLDiagramItem
    * @see         FSAContainer
    * @see         FSAObject#getPropertyName
    */
   public static FSAContainer getMainFSAContainer (ASGElement item)
   {
      return (FSAContainer) getFSAObject (item, UnparseManager.get().getUnparseModule (item).getMainFsaName());
   }


   /**
    * Get item's FSAObject with given propertyName.
    *
    * @param item          The UMLDiagramItem's FSAObjects are searched.
    * @param propertyName  identifies item's FSAObject.
    * @return              The item's FSAObject with given propertyName.
    * @see                 UMLDiagramItem
    * @see                 FSAObject
    * @see                 FSAObject#getPropertyName
    */
   public static FSAObject getFSAObject (ASGElement item,
                                         String propertyName)
   {
      // Walk through all FSAObjects and return the first one with a
      // propertyName == "propertyName"
      FSAObject object = null;
      Iterator iter = item.iteratorOfFsaObjects();
      while (iter.hasNext() && object == null)
      {
         FSAObject i = (FSAObject) iter.next();
         if (i.getPropertyName().equals (propertyName))
         {
            object = i;
         }
      }
      return object;
   }



   /**
    * Get the usecaseDiagramOfUsecase attribute of the Utility class
    *
    * @param usecase  No description provided
    * @return         The usecaseDiagramOfUsecase value
    */
   public static UsecaseDiagram getUsecaseDiagramOfUsecase (Usecase usecase)
   {
      UsecaseDiagram diag = null;
      if (usecase.sizeOfDiagrams() == 1)
      {
         diag = (UsecaseDiagram)  ((UsecaseSystemDiagram) usecase.getFirstFromDiagrams()).getSystem().getFirstFromDiagrams();
      }
      else
      {
         throw new RuntimeException ("INTERNAL ERROR: Usecase "
            + usecase + " in more than one UMLDiagram!");
      }
      return diag;
   }


   /**
    * Get the diagramOfDiagramItem attribute of the Utility class
    *
    * @param item  No description provided
    * @return      The diagramOfDiagramItem value
    */
   public static UMLDiagram getDiagramOfDiagramItem (UMLDiagramItem item)
   {
      UMLDiagram diag = null;
      if (item.sizeOfDiagrams() == 1)
      {
         diag = item.getFirstFromDiagrams();
      }
      else
      {
         throw new RuntimeException ("INTERNAL ERROR: UMLDiagramItem "
            + item + " in more than one UMLDiagram!");
      }
      return diag;
   }



   /**
    * Get the singleSelectedUsecase attribute of the Utility class
    *
    * @return            The singleSelectedUsecase value
    * @throws Exception  Exception description not provided
    */
   public static Usecase getSingleSelectedUsecase() throws Exception
   {
      UMLDiagramItem item = getSingleSelectedDiagramItem();

      Usecase usecase = null;
      if (item instanceof Usecase)
      {
         usecase = (Usecase) item;
      }
      else
      {
         throw new Exception ("Select a Use Case, please.");
      }

      return usecase;
   }



   /**
    * Get the singleSelectedDiagramItem attribute of the Utility class
    *
    * @return            The singleSelectedDiagramItem value
    * @throws Exception  Exception description not provided
    */
   public static UMLDiagramItem getSingleSelectedDiagramItem() throws Exception
   {
      // there must only be one selected UMLDiagramItem
      Iterator iter
          = SelectionManager.get().iteratorOfSelectionAsIncrements();
      UMLIncrement incr;

      if (iter.hasNext())
      {
         incr = (UMLIncrement) iter.next();
      }
      else
      {
         throw new Exception ("Select an item, please.");
      }
      if (iter.hasNext())
      {
         throw new Exception ("Select only one item, please.");
      }

      UMLDiagramItem item = null;
      if (incr instanceof UMLDiagramItem)
      {
         item = (UMLDiagramItem) incr;
      }
      else
      {
         throw new Exception ("Select an item, please.");
      }

      return item;
   }



   /**
    * Get the usecaseDiagramByName attribute of the Utility class
    *
    * @param name  No description provided
    * @return      The usecaseDiagramByName value
    */
   public static UsecaseDiagram getUsecaseDiagramByName (String name)
   {
      UsecaseDiagram result = null;

      Iterator iter = UMLProject.get().iteratorOfDiags();
      while (iter.hasNext() && result == null)
      {
         ASGDiagram umldiag = (ASGDiagram) iter.next();
         if (umldiag instanceof UsecaseDiagram)
         {
            UsecaseDiagram diag = (UsecaseDiagram) umldiag;
            if (diag.getName().equals (name))
            {
               result = diag;
            }
         }
      }

      return result;
   }
}

/*
 * $Log: Utility.java,v $
 * Revision 1.15  2004/10/20 17:50:28  schneider
 * Introduction of interfaces for class diagram classes
 *
 */
